/*
  DC Motor Speed Control (FA-130RA-2270)

  IDE: Energia (http://energia.nu/)

  BORAD: MSP-EXP430G2
  MCU: M430G2553
    U$2: USB (5V supply)
    TP1: VCC5 (5V)
    J6:1: VCC3 (3.6V)
    J6:3: GND
    J2:19: PWM output (TA0.1)
    J1:6: EMF reference (A4)
    J1:7: EMF feedback (A5)
    S2: ON/OFF switch (P1.3)
    LED1(green): PWM indicator (TA0.1)
    LED2(red): alive indicator 1s blink (P1.6)
 */

static const uint8_t volumePin = A4;    // select the input pin for the potentiometer
static const uint8_t emfPin = A5;    // select the input pin for the potentiometer
static const uint8_t motorPin = P2_6; // select the pin for the LED
static const uint8_t buttonPin = PUSH2;     // the number of the pushbutton pin

static int16_t emf = 0;
static int16_t volume = 0;
static int16_t om = 0;
static int16_t omRef = 0;
static int32_t omErr = 0;
static int32_t mRef_p = 0;
static int32_t mRef_isum = 0;
static int32_t mRef_i = 0;
static int32_t mRef_pi = 0;
static int32_t mRef = 0;
static int32_t mRef_over = 0;
static uint16_t startTime = 0;
static uint16_t endTime = 0;

static uint8_t isOn = 0;

void setup() {
  Serial.begin(9600); 
  pinMode(GREEN_LED, OUTPUT);
  pinMode(RED_LED, OUTPUT);
  pinMode(motorPin, OUTPUT);
  analogFrequency(10000);
  analogResolution(1023);
  analogWrite(motorPin, 1023 - 1);
  analogWrite(GREEN_LED, 1023 - 1);
//  TA0CCTL0 |= CCIE; /* CCR0 interrupt enabled */

  /* Initialize Timer_A0 */
  TA1CCTL0 = CCIE; /* CCR0 interrupt enabled */
  TA1CCR0 = F_CPU / 1000; /* 1000Hz */
  TA1CTL = TASSEL_2 | MC_1; /* SMCLK, upmode */

  pinMode(buttonPin, INPUT_PULLUP);
  attachInterrupt(buttonPin, onoff, FALLING); // Interrupt is fired whenever button is pressed
}

void loop() {
  Serial.print(om);
  Serial.print(", ");
  Serial.print(omRef);
  Serial.print(", ");
  Serial.print(mRef_p);
  Serial.print(", ");
  Serial.print(mRef_i);
  Serial.print(", ");
  Serial.print(mRef);
  Serial.print(", ");
  Serial.print(endTime - startTime);
  Serial.print(", ");
  Serial.println(isOn);
  delay(1000);
}

/* Timer0 A0 interrupt service routine */
__attribute__((interrupt(TIMER0_A0_VECTOR)))
void TIMER0_A0_ISR ( void ) {
}

/* Timer1 A0 interrupt service routine */
__attribute__((interrupt(TIMER1_A0_VECTOR)))
void TIMER1_A0_ISR ( void ) {
  startTime = TA1R;
  emf = analogRead(emfPin);
  volume = analogRead(volumePin);

  om = emf * 2;
  omRef = volume;

  if (isOn) {
    omErr = omRef - om;
    mRef_p = 256 * omErr;
    mRef_isum += mRef_p - mRef_over;
    mRef_i = mRef_isum / 256;
    mRef_pi = mRef_p + mRef_i;
    mRef = constrain(mRef_pi, 0, 1023);
    mRef_over = mRef_pi - mRef;
  }
  else {
    omErr = 0;
    mRef_p = 0;
    mRef_isum =0;
    mRef_i = 0;
    mRef_pi = 0;
    mRef = 0;
    mRef_over = 0;
  }
  
  analogWrite(motorPin, 1023 - mRef);
  analogWrite(GREEN_LED, 1023 - mRef);

  blinkLED();
  endTime = TA1R;
}

void blinkLED( void ) {
  static uint16_t counter = 0;
  counter++;
  if (counter >= 1000) {
    digitalWrite(RED_LED, !digitalRead(RED_LED));
    counter = 0;
  }
}

void onoff( void ) {
  isOn = !isOn;
}

